-- -- --
--
-- Daily Quest Count-and-Track Addon
--
-- MacGregor Personal Stubs
--  Corwin MacGregor
--
-- -- --
--
-- Track Daily Quests completed, post-p5.0.4
--
-- -- --

---------------------
-- Local Variables --
---------------------
local addonName = "DailyQuestTracker"
local addon  = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceHook-3.0" )

local QLF 	= QuestLogFrame;
local QLC 	= QuestLogCount;
local QLQC 	= QuestLogQuestCount;
local DQC 		-- QuestLogDailyQuestCount
local DQC_MOF 	-- QuestLogDailyQuestCountMouseOverFrame

local QUEST_LOG_DAILY_COUNT_TEMPLATE = gsub(QUEST_LOG_DAILY_COUNT_TEMPLATE, "%%d", "%%3d");
local QUEST_LOG_DAILY_TOOLTIP = gsub(QUEST_LOG_DAILY_TOOLTIP, "%%d Daily quests", "%%d daily |4quest:quests;");
  -- may need to consider a more rigorous gsub() for QUEST_LOG_DAILY_TOOLTIP, to account for localizations

---------------------
-- Addon Handlers  --
--------------------- 

function addon:OnInitialize()

	-- re-justify (to the left) the Quest Count  (v1.2)
	QLQC:ClearAllPoints();
	QLQC:SetPoint("TOPLEFT", QLC.BorderTopLeft, "BOTTOMRIGHT", 0, 3);

	-- create and justify-right the Daily Quest Count
	DQC = QLC:CreateFontString("QuestLogDailyQuestCount", "ARTWORK", "GameFontNormalSmall");
	DQC:SetPoint("TOPRIGHT", QLC.BorderTopRight, "BOTTOMLEFT", 1, 3);
	-- hook the update.  post-hook, since this code was run after what is left, anyways.
	self:SecureHook("QuestLog_UpdateQuestCount", QuestLog_UpdateDailyQuestCount);

	-- Create and Initialize Daily Quest Counter's MouseOver Frame (v1.1)
	do
		DQC_MOF = CreateFrame("Frame", "QuestLogDailyQuestCountMouseOverFrame", QLC);
		DQC_MOF:EnableMouse("true");
		DQC_MOF:SetSize("5", "12");
		DQC_MOF:SetPoint("LEFT", "QuestLogDailyQuestCount");
		DQC_MOF:SetPoint("RIGHT", "QuestLogDailyQuestCount");
		DQC_MOF:SetScript("OnEnter",
			function (...)
				GameTooltip:SetOwner(DQC_MOF, "ANCHOR_RIGHT");
				GameTooltip:SetText(format(QUEST_LOG_DAILY_TOOLTIP, GetDailyQuestsCompleted(), SecondsToTime(GetQuestResetTime(), nil, 1)));
			end )
		DQC_MOF:SetScript("OnLeave", GameTooltip_Hide)
	end
end


function addon:OnEnable()
end


function addon:OnDisable()
end


---------------------
-- Addon Functions --
--------------------- 

function QuestLog_UpdateDailyQuestCount(...)
	local textHeight = 12;
	local hPadding = 15;
	local vPadding = 8;
	local width = QLQC:GetWidth();

	DQC:SetFormattedText(QUEST_LOG_DAILY_COUNT_TEMPLATE, GetDailyQuestsCompleted());
	DQC:Show();
	DQC_MOF:Show();

	-- set the background platterframe
	QLC:SetHeight(textHeight+vPadding);
	QLC:SetPoint("TOPLEFT", QLC:GetParent(), "TOPLEFT", 70, -33);
	QLC:SetWidth(QLQC:GetWidth()+DQC:GetWidth()+40+hPadding);
end

--[[
--]]


--[[------------------
--  Project Notes   --
----------------------

Quest Log Daily Quest Counter Stub
  v0.0 = initial pull-down of old FrameXML code
  v1.0 = text label indicating daily quest completed count
  v1.1 = mouseover tooltip indicating reset time
  v2.0 = scrollable tooltip indicating dailies completed

p4.3.4	QuestLogFrame.xml
		-> QuestLogCount:  QuestLogDailyQuestCount
		QuestLogFrame.lua
		-> QuestLog_UpdateQuestCount()

p5.0.4	QuestLogFrame.xml
		-> QuestLogCount
		QuestLogFrame.lua
		->QuestLog_UpdateQuestCount()

* dynamically add FontString QuestLogDailyQuestCount to QuestLogCount's ARTWORK layer
-->>	QuestLogCount:CreateFontString("QuestLogDailyQuestCount", "ARTWORK", "GameFontNormalSmall");
-->>	QuestLogDailyQuestCount:SetPoint("TOPLEFT", "QuestLogQuestCount", "BOTTOMRIGHT", "0", "-2");

* post-hook QuestLog_UpdateQuestCount() with old daily quest count updating code

-->>
function QuestLog_UpdateDailyQuestCount()
	local textHeight = 12;
	local hPadding = 15;
	local vPadding = 8;
	local dailyQuestsComplete = GetDailyQuestsCompleted();
	local parent = QuestLogCount:GetParent();
	local width = QuestLogQuestCount:GetWidth();

	if ( dailyQuestsComplete > 0 ) then
		QuestLogDailyQuestCount:SetFormattedText(QUEST_LOG_DAILY_COUNT_TEMPLATE, dailyQuestsComplete);
		QuestLogDailyQuestCount:Show();
		QuestLogDailyQuestCountMouseOverFrame:Show();
		if ( QuestLogDailyQuestCount:GetWidth() > width ) then
			width = QuestLogDailyQuestCount:GetWidth();
		end
		QuestLogCount:SetHeight(textHeight*2+vPadding);
		QuestLogCount:SetPoint("TOPLEFT", parent, "TOPLEFT", 80, -38);
	else
		QuestLogDailyQuestCount:Hide();
		QuestLogDailyQuestCountMouseOverFrame:Hide();
		QuestLogCount:SetHeight(textHeight+vPadding);
		QuestLogCount:SetPoint("TOPLEFT", parent, "TOPLEFT", 80, -41);
	end
	QuestLogCount:SetWidth(width+hPadding);
end
<<--

--
p4.3.3	QuestLogFrame.xml
		->QuestLogCount:  QuestLogDailyQuestCountMouseOverFrame

-->>
	DQC_MOF = CreateFrame("Frame", "QuestLogDailyQuestCountMouseOverFrame", "QuestLogCount");
	DQC_MOF:EnableMouse("true");
	DQC_MOF:SetSize("5", "12");
	DQC_MOF:SetPoint("LEFT", "QuestLogDailyQuestCount");
	DQC_MOF:SetPoint("RIGHT", "QuestLogDailyQuestCount");
	DQC_MOF:SetScript("OnEnter",
		function (...)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(format(QUEST_LOG_DAILY_TOOLTIP, GetDailyQuestsCompleted(), SecondsToTime(GetQuestResetTime(), nil, 1)));
		end )
	DQC_MOF:SetScript("OnLeave", GameTooltip_Hide)
<<--
	<Frame name="QuestLogDailyQuestCountMouseOverFrame" enableMouse="true">
		<Size>
			<AbsDimension x="5" y="12"/>
		</Size>
		<Anchors>
			<Anchor point="LEFT" relativeTo="QuestLogDailyQuestCount"/>
			<Anchor point="RIGHT" relativeTo="QuestLogDailyQuestCount"/>
		</Anchors>
		<Scripts>
			<OnEnter>
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(format(QUEST_LOG_DAILY_TOOLTIP, GetDailyQuestsCompleted(), SecondsToTime(GetQuestResetTime(), nil, 1)));
			</OnEnter>
			<OnLeave function="GameTooltip_Hide"/>
		</Scripts>
	</Frame>

--

/run DEFAULT_CHAT_FRAME:AddMessage(format("# %d \1244Daily quest:Daily quests;... #", 1))

-- -- --

v2.0 Track dailies already completed

OnEvent QUEST_ACCEPTED -- arg1 ::= questlog index
	local title, _, _, _, _, _, _, isDaily, questID = GetQuestLogTitle(arg1);
	if isDaily then
		watches[questID].name = title;
		watches[questID].progress = 1;
	end
end

OnEvent QUEST_LOG_UPDATE
	if isUpdating then return; end
	isUpdating = true;

	foreach index in questlog do
		local title, _, _, _, _, _, isComplete, isDaily, questID = GetQuestLogTitle(index);
		if isDaily and watches[questID] then
			if isComplete and isComplete < 0 then  -- quest failed
				watches[questID].progress = -1;
			elseif isComplete and isComplete > 0 then -- quest objectives completed
				watches[questID].progress = 2;
			end
		end
	end
end

#POSTHOOK_OnFuncCall GetQuestReward()
	local questID = GetQuestID();
	if watches[questID] then
		watches[questID].progress = 3;
	end
end

-- -- --
v2.5 accept dailies and track their progress.  indicate dependancies

DailyQuest Dependancy DB format

DB ::= {<questID>=<questDeps>, ...}
questDeps ::= {<questDep>, ...}
questDep ::= <questID> | <randomQuestDepGroup>
randomQuestDepGroup ::= {<questID>, <questID>, ...}  -- N.B.:  Have to have at least two quests in random dep group

--> will need a Master Dependancies DB?
--> without a GetQuestInfo(), how will we pull quest names via questID?  D:
--> will need a master dependancies DB< and also a cross-reference DB maintaining questID.title

---------------------
--   Change  Log   --
---------------------
-- version format:  MAJOR.MINOR
-- date stamp


v1.0  [09/05/2012]
*  initial work
*  fixed layout formatting to fit with new QuestLogFrame
*  RELEASABLE

v1.1  [09/05/2012]
*  restablish QuestLogDailyQuestCountMouseOverFrame
*  cleaned up old code
*  RELEASABLE

v1.2  [09/06/2012]
*  reformatted layout of the two counts into a single line
*  RELEASABLE

v1.2b	[09/12/2012]
*  corrected formatting of QUEST_LOG_DAILY_TOOLTIP
*  RELEASABLE

-- -- --

TO DO:

*  2.0 capture daily quest names as completed, display list
   ->  scrollable tooltip through DQC_MOF, a la Archy?
   ->  hijack QuestLogScrollFrame with tab-like behavior?  (perhaps add keybinding? SHIFT-Questlog)

*  2.5 load daily quests onto DQT list by quest ID
   ->  color-code by progress:
   grey	 0	not acquired
   yellow	 1	in progress
   green 	 2	objectives completed
   red	-1	failed
   white	 3	turned in
   orange	-2	dependancy locked (re: Molten Front dailies)

--> new tactic:  as daily quest is acquired, provide toggle in log to track progress of quest even in the future

-------------------]]
-- // --